%Start header st dt inc quote
%e 1300
%k 100
%a 1400
%o 1200
%p 4000
%n 600
%{

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <malloc.h>
#include "sp.h"
#include "y.tab.h"

FILE		*fin;
int input1(){
	register int c;
	c = getc(fin);
	if(isupper(c)) c = tolower(c);
	return c == EOF ? 0 : c;
}
int unput1(c) { return ungetc(c,fin);}

int		linenum;
char	*filename;
# define	INCBUFSIZE	100
static		char		incbuf[INCBUFSIZE];
char		*title;
extern char	*savestr();
static struct {
	char *name;
	int  id;
} lexkw[] = {
	"ac",	AC,
	"dc",	DC,
	"end",	END,
	"ends",	ENDS,
	"four", FOUR,
	"global", GLOBAL,
	"ic", IC,
	"include", INCLUDE,
	"lib", LIB,
	"model", MODEL,
	"nodeset", NODESET,
	"noise", NOISE,
	"op", OP,
	"option", OPTIONS,
	"options", OPTIONS,
	"opt", OPTIONS,
	"param", PARAM,
	"pr",    PR,
	"print", PR,
	"sens",  SENS,
	"subckt",  SUBCKT,
	"temp",  TEMP,
	"tf",    TF,
	"tr",    TR,
	"width", WIDTH,
	(char *)0,      UNKNOWN
};
static int printtok = 0;
static unsigned char strcmp_spice(char *s1,char *s2){
	while(*s1 || *s2){
		if(*s1 == *s2 || (*s1 - 'A' + 'a' == *s2)){
			s1++;s2++;
		}
		else
			return 1;
	}
	return 0;
}
static int finddotkeyword(s) char	*s;
{
	int i;
	for(i=0; lexkw[i].name; i++)
		if(!strcmp_spice(s, lexkw[i].name)) break;
	return lexkw[i].id;
}
%}
A	[a-zA-Z_~]
B	[a-zA-Z0-9_\.\+\-]
D       [0-9]
WS      [=(),; \t]+
W		[ \t]+
C      \n{W}?"+"
SQ      '[^']+'
DQ      \"[^\"]+\"
%%

	switch (yy_start) {
		case 0:
    		BEGIN header;
			break;

	}

<header>.*$     {  BEGIN st; linenum++; title = strdup(&yytext[0]);}
<st>"."{A}{B}*	{ int key;
					BEGIN dt;
					switch(key=finddotkeyword(yytext +1)){
				
					case UNKNOWN :
						fprintf(stderr,"*WARNING: unknown control keyword \"%s\", skipping. at line:%ld \n", yytext+1,linenum+1);
					case AC : case DC :
					case FOUR : case IC :
					case NODESET : case NOISE :
					case OP : case OPTIONS :
					case PARAM : case PR :
					case SENS : case TEMP :
					case TF : case TR :
					case WIDTH :
					default :
						yylval.keyword = key;
						return CONTROL;
					case END : case ENDS :
					case GLOBAL : case SUBCKT : 
					case LIB : case MODEL:
						printtok=1;
						return key;
					case INCLUDE :
						BEGIN inc;
						break;
					}
				}
<st>{A}{B}* 	{ 
					BEGIN dt;
					yylval.p = savestr(yytext);
					return yytext[0];
				}
<inc>'[^']+'	{	yytext[yyleng-1] = (char)0;
					strcpy(&incbuf[0], yytext+1);
				}
<inc>{A}+		{ yyerror("wrong include syntax!"); }
<dt>{SQ}        { yytext[yyleng-1] = (char)0;
				  yylval.p = savestr(yytext+1);
				  return DATUM;
				}
<dt>{DQ}        { yytext[yyleng-1] = (char)0;
				  yylval.p = savestr(yytext+1);
				  return DATUM;
				}
<dt>{B}*		{ yylval.p = savestr(yytext); return DATUM;}
<dt>[\$\*].*$		;	/* comments beginning with $ * */
{WS}			;	/* white space */

<st,dt>\n[\+] {

			linenum++;
			/*Continue this token further*/
			}

<st,dt>\n[\*].*$	{
			linenum++;
	}
<st,dt>\n  { 
			BEGIN st;
			return EOS;
	}
<inc>\n[\+] {
			linenum++;
	}
<inc>\n[\*] {
		linenum++;
	}

<inc>\n {
	BEGIN st;
	yypushnewfile(&incbuf[0]);
	}

%%
typedef struct inputs Inputs;
struct inputs {
	char	*name;
	FILE	*fp;
	int		linenum;
};
# define	OPENFILES	15
Inputs	filestack[OPENFILES];
Inputs	*filestackp = &filestack[0];
yypushnewfile(s)
char	*s;
{
	filestackp->name = filename;
	filestackp->fp = fin;
	filestackp->linenum = linenum;
	filestackp++;
	linenum = 1;
	filename = strdup(s);
	fin = fopen(s,"r");
	if(!fin){
		fprintf(stderr,"can not read \"%s\"\n",filename);
		exit(1);
	}
}
yywrap()
{
		fclose(fin);
		filestackp--;
		if(filestackp >= &filestack[0]){
			fin = filestackp->fp;
			free(filename);
			filename = filestackp->name;
			linenum = filestackp->linenum;
			return 0;
		}
		return 1;
}
yyerror(s)
char	*s;
{
	fprintf(stderr,"%s[%d] %s near \"%s\"\n",filename, linenum,s, yytext); 
}
double spatof(register char *s)
{
	extern	double atof();
	double d;
	d = atof(s);
	while(isdigit(*s) || *s == '.' || *s=='+' || *s=='-' || *s == 'e') s++;
	switch(*s)
	{
	case 'f' : d *= 1.0e-15; break;
	case 'p' : d *= 1.0e-12; break;
	case 'n' : d *= 1.0e-9; break;
	case 'u' : d *= 1.0e-6; break;
	case 'm' :
		if(s[1] == 'i') d *= 25.4e-6;
		else if(s[1] == 'e') d *= 1.0e6;
		else d *= 1.0e-3; break;
	case 'k' : d *= 1.0e3; break;
	case 'g' : d *= 1.0e9; break;
	case 't' : d *= 1.0e12; break;
	}
	return d;
}
