/**
 * Yacc file for parsing the technology mapped netlist 
 * from the design compiler.
 * vamsi.krishnak@gmail.com Mar 29 2007
 */
%{
#include "List.h"
#include "hash.h"
#include <assert.h>
#include "verilog.h"
#include <string.h>
extern int my_strcmp(char *,char *);
	
static List *arglist = NULL;
static List *argptr = NULL;
static char wire_statement_scan = 0;
extern unsigned int current_line_no;
char libmode = 0;
char *current_mod_name;
Module *current_mod_ptr = NULL;
%}
%union {
	char *str;
	int num;
	List *list;
	PinPair *ppair;
}
%token <str> WORD
%token <str> IDENTIFIER
/*Key Words*/
%token <str> MODULE_KEY_WORD
%token <str> ENDMODULE_KEY_WORD
%token <str> OUTPUT_KEY_WORD
%token <str> INPUT_KEY_WORD
%token <str> ASSIGN_KEY_WORD
%token <str> WIRE_KEY_WORD

%token <str> STRING /*any thing except a ';'*/
%token <str> NEW_LINE
%token <str> SEMI_COLON
%token <str> DOT
%token <str> LEFT_BRACE
%token <str> RIGHT_BRACE
%token <str> COMMA
%token <str> ASSIGN_OPERATOR
/*Type definitions for the values of the rules*/
%type <list> termlist
%type <list> arguments
%type <list> pinpairarguments
%type <ppair> pinpair
%type <list> arglist
%type <str> modsignature
%%
verilog: verilog module | module  
module: modsignature modbody modend 
	{
#ifdef PF_VERBOSE
		printf("MODULE SCANNED: %s SUCCESS\n",$1);
#endif

	} 
modsignature: MODULE_KEY_WORD IDENTIFIER arglist SEMI_COLON 
			{
				Module *m = CreateNewModule($2,$3,NULL);
				m->mod_name = $2;
				current_mod_name = $2;
				current_mod_ptr = m;
				$$ = $2;
			}
arglist: 
		{
			$$ = NULL;
	    }
		| LEFT_BRACE termlist RIGHT_BRACE 
		{
			$$ = $2;
		}

termlist: pinpairarguments 
		  {
			  
			  /*List *ptr = $1;
			  PinPair *ptr1;
			  while(ptr){
				  ptr1 = (PinPair *) ptr->_data;
				  printf("PinPair .%s(%s)\n",
				  	ptr1->block_pin,ptr1->net_pin);
				  ptr = ptr->next;
			  }*/
			  $$ = $1;
		  }
		  | arguments 
		  { 
			  /*printf("Printing the ARGLIST.......\n");
			  PrintList_String($1);*/
			  $$ = $1;
		  }

pinpair: DOT IDENTIFIER LEFT_BRACE IDENTIFIER RIGHT_BRACE 
		{
			PinPair *ptr = (PinPair *) malloc(sizeof(PinPair)*1);
			ptr->block_pin = $2;
			ptr->net_pin = $4; 
			$$ = ptr;
		}
pinpairarguments: pinpair 
				  {
			  		argptr = NewListNode();
			  		argptr->_data = $1; 
			  		argptr->next = NULL;
			  		$$ = argptr;
				  } 
				 | pinpair COMMA pinpairarguments 
				 {
			  		argptr = NewListNode();
			  		argptr->_data = $1;
			  		argptr->next = $3;
			  		$$ = argptr;
				 }
arguments: IDENTIFIER 
		   {
			   if(!wire_statement_scan){
			  		argptr = NewListNode();
			  		argptr->_data = (void *)$1; 
			  		argptr->next = NULL;
			  		$$ = argptr;
			   }else{
				   free($1);
				   $$ = NULL;
			   }
		   } 
	       | IDENTIFIER COMMA arguments
		   {
			   if(!wire_statement_scan){
			  		argptr = NewListNode();
			  		argptr->_data = $1;
			  		argptr->next = $3;
			  		$$ = argptr;
			   }else{
				   free($1);
				   $$ = NULL;

			   }
		   }
modend: ENDMODULE_KEY_WORD

/*A module must at atleast 1 input and 1 output statement*/
modbody: statement | statement modbody

statement: inputstatement | outputstatement 
		   | gatestatement | wirestatement | assignstatement
 
inputstatement: INPUT_KEY_WORD arguments SEMI_COLON {
	if(!SetPinType(current_mod_ptr,$2,IN_PIN)){
		exit(1);
	}
}
outputstatement: OUTPUT_KEY_WORD arguments SEMI_COLON { 
	if(!SetPinType(current_mod_ptr,$2,OUT_PIN)){
		exit(1);
	}
}
gatestatement: IDENTIFIER IDENTIFIER LEFT_BRACE termlist RIGHT_BRACE SEMI_COLON {
	if(!strcmp(expand_this_cell,current_mod_name)){
#if PF_VERBOSE
		printf("Scanning gate statement\n");
#endif
		Module *def = (Module *) hfetch(mod_def,$1);
		if(!def){
			fprintf(stderr,"Unable to find Module definition of %s at line %d\n",
				$1,current_line_no);
			exit(1);
		}
#if PF_VERBOSE
		printf("Module %s definition found\n",$1);
#endif

		List *ptr = $4;
		List *i,*j;
		NetList *net_list_ptr = NULL;
		if(!ptr){
			printf("PinPair list is NULL\n");
		}
	    PinPair *ptr1;
		Element *e = CreateNewElement($2,def,current_mod_ptr);
		/*Create a NetList for each of this 
		 *nets if one is not already created.*/
	    while(ptr){
		  ptr1 = (PinPair *) ptr->_data;
		  /*printf("PinPair .%s(%s)=%s\n",
	      ptr1->block_pin,ptr1->net_pin,hfetch(def->nets,(char *)ptr1->block_pin));*/
			/*Lets Store Net2Pin Pin2Net mapping*/
			hstore(e->net_list,ptr1->block_pin,ptr1->net_pin);
			if(strcmp(ptr1->block_pin,ptr1->net_pin)){
				hstore(e->net_list,ptr1->net_pin,
					ptr1->block_pin);
			}

		  net_list_ptr = hfetch(current_mod_ptr->net_list,(char *)ptr1->net_pin);
		  if(!net_list_ptr){
			  net_list_ptr = (NetList *) malloc(sizeof(NetList)*1);
			  assert(net_list_ptr != NULL);
			  net_list_ptr->net_start = NULL;
			  net_list_ptr->net_end = NULL;
			  net_list_ptr->size = 0;
			  hstore(current_mod_ptr->net_list,(char *)ptr1->net_pin,(void *) net_list_ptr);
		  }
		  net_list_ptr->size++;
		  InsertList(FIRST,&(net_list_ptr->net_start),e);
		  /*Put this pin in the output/input fanin_nets/fanout_nets of e*/
		  if(!strcmp(hfetch(def->nets,(char *)ptr1->block_pin),"INPUT")){
		  		InsertList(FIRST,&(e->fanin_nets),(void *)ptr1->net_pin);
		  }else{
		  		InsertList(FIRST,&(e->fanout_nets),(void *)ptr1->net_pin);
		  }
		  ptr = ptr->next;
	    }
	    InsertList(FIRST,&(current_mod_ptr->elements),e);
	}
}
wirestatement: WIRE_KEY_WORD {wire_statement_scan=1;} arguments SEMI_COLON /*| error SEMI_COLON*/{
	printf("Wire statement found::IGNORED \n");
	wire_statement_scan=0;
}
assignstatement: ASSIGN_KEY_WORD IDENTIFIER ASSIGN_OPERATOR IDENTIFIER SEMI_COLON {

	if(!strcmp(expand_this_cell,current_mod_name)){
		NetList *ptr = hfetch(current_mod_ptr->net_list,$4);
		if(!ptr){
			ptr = (NetList *)malloc(sizeof(NetList)*1);
			ptr->net_start = NULL;
			ptr->net_end = NULL;
			hstore(current_mod_ptr->net_list,$4,(void *)ptr);
		}
		/*Set the netlist same for both the guys*/
		hstore(current_mod_ptr->net_list,$2,(void *)ptr);
	}
}
%%
#include<stdio.h>
int yywrap(){
	return 1;
}
extern unsigned int current_col_no;
extern unsigned int current_line_no;

int yyerror(char *yymsg){
	fprintf(stderr,"Parse error....%s\n",yymsg);
	fprintf(stderr,"At line #%ld-%ld\n",current_line_no,current_col_no);
}

List *NewListNode(){
	List *ptr = NULL;
	ptr = (List *) malloc(sizeof(List)*1);
	assert(ptr != NULL);
	return ptr;
}
