#include "path_algorithm.h"
#include "verilog.h"
#include <assert.h>
#include <stdlib.h>
#include "util.h"
#include <stdio.h>
#include <assert.h>
extern char *memory_module_name;
HASH *TFaultHash = NULL;
FILE *TFaultFile = NULL;
extern char *start_mem_cell_name;
extern long path_size;
status_t OpenTFaultFile(const char *file_name){
	TFaultFile = fopen(file_name,"r");
	if(TFaultFile == NULL){
		fprintf(stderr,"Unable to open %s for reading\n",file_name);
		return 0;
	}
	return 1;
}
/**
 * vamsi.krishnak@gmail.com Apr 21, 2007 Created.
 */
/*AddPath2Element: Add the edge v1->instance_name,pin_v1 to each of
 */
void AddPath2Element(Element *v1,Element *v2,char *pin_v1){
	PathList *ptr = v1->path_list;
	char *current_path=NULL;
	char *new_path=NULL;
	long length1,length;
	assert(pin_v1 != NULL);
	if(!ptr){
	/*Just add the edge*/
		length = strlen(v1->instance_name) + strlen(pin_v1)+2;
		new_path = (char *) malloc(sizeof(char)*length);
		sprintf(new_path,"%s/%s",v1->instance_name,pin_v1);
		InsertList(FIRST,&(v2->path_list),(void *)new_path);
		return;
	}

	while(ptr){
		/*We are not supposed to have a path which is NULL*/
		assert(ptr->_data != NULL);
		length1 = strlen((char *)ptr->_data);
		assert(length1 > 0);
		length = length1+strlen(v1->instance_name) + strlen(pin_v1);
		length += 3;
		new_path = (char *) malloc(sizeof(char)*length);
		assert(new_path != NULL);
		new_path[length] = '\0';
		/*TODO: Need to check the return values of sprintf*/
		sprintf(new_path,"%s %s/%s",(char *)ptr->_data,v1->instance_name,pin_v1);
		InsertList(FIRST,&(v2->path_list),(void *)new_path);
		ptr = ptr->next;
	}
}
static char temp_buffer[512];
static List *GetUnMarkedElements(Element *e){
	List *ptr = e->fanout_nets;
	List *new_list = NULL;
	NetList *net_list = NULL;
	char *pin_name = NULL;	

	List *i;
	if(ptr == NULL){
		fprintf(stderr,"WARNING: %s has no fanouts\n",
			e->instance_name);
		return NULL;
	}


	while(ptr){
		net_list = (NetList *) hfetch((e->parent_cell)->net_list,
			(char *)ptr->_data);
		if(!net_list){
			fprintf(stderr,"Netlist is NULL for net %s\n",
				(char *)ptr->_data);
		}
		i = net_list->net_start;
		assert(TFaultHash != NULL);
		while(i){
			/*First check if this is in TFaultHash*/
			pin_name = hfetch(((Element *)i->_data)->net_list,
				(char *)ptr->_data); 
			assert(pin_name != NULL);
			sprintf(temp_buffer,"%s/%s",
				((Element *)i->_data)->instance_name,pin_name);
			/*We need to check if this is a TFault*/
			if(!hfetch(TFaultHash,temp_buffer)){
				i = i->next;
				continue;
			}

			if(!(((Element *)i->_data)->Marked)){
				InsertList(FIRST,&new_list,(void *)i->_data);
				hstore(((Element *)i->_data)->net_list,
					"__pathfinder__connecting_net",
					(char *)ptr->_data);
			}
			i = i->next;
		}
		ptr = ptr->next;
	}
	return new_list;
}



/*Free Paths per element*/
void FreePaths(Element *e){
	PathList *ptr = e->path_list;
	FreeList((List *)ptr,free);
	/*e->path_list has been freed.*/
	e->path_list = NULL;
}

void FreePathsAllElements(Module *m){
	List *e_list = m->elements;
	assert(e_list != NULL);
	while(e_list){
		FreePaths((Element *)e_list->_data);
		((Element *)e_list->_data)->Marked = 0;
		e_list = e_list->next;
	}
}
/*If TFaultHash is not yet created then it creates
 **/
status_t CreateTFaultHash_File(FILE *fptr){
	char line_buffer[512];
	char buf[64],buf1[64];
	if(!TFaultHash){
		TFaultHash = hnew();
		assert(TFaultHash!=NULL);
	}
	assert(fptr!=NULL);
	while(fscanf(fptr,"%s%s%s",buf,buf1,line_buffer)==3){
		if(!hfetch(TFaultHash,line_buffer)){
			hstore(TFaultHash,line_buffer,"1");
		}
	}
	return SUCCESS;
}

/*
 *Create's a TFault Hash as follows
 * Key == InstanceName/PinName
 */
static status_t CreateTFaultHash(List *tf_list){
	List *ptr = tf_list;
	TFault *tf;
	char *keybuf;
	int length,length1;
	if(!TFaultHash){
		TFaultHash = hnew();
		assert(TFaultHash != NULL);
	}
	while(ptr){
		tf = (TFault *)tf_list->_data;
		length = strlen(tf->instance_name); 
		assert(length > 1);
		length1 = strlen(tf->pin_name);
		assert(length1 > 1);
		keybuf = (char *) malloc(sizeof(char)*(length+length1+1));

		if(strncpy(keybuf,tf->instance_name,length) != keybuf){
			fprintf(stderr,"CreateTFaultHash:: FAILED\n");
			return FAILURE;
		}
		if(strncpy((keybuf+length),tf->pin_name,length1) != (keybuf+length)){
			fprintf(stderr,"CreateTFaultHash:: FAILED\n");
			return FAILURE;
		}
		keybuf[length+length1+1] = '\0';
#ifndef NDEBUG
		printf("TFaultHash::Created %s\n",keybuf);
#endif
		
		hstore(TFaultHash,keybuf,"1");
		ptr = ptr->next;
	}
}

/**
 * A smart list appender
 */
void AppendToList(List **list_ptr, List *new_list){
	static List **prev_list = NULL;
	static List *tail_ptr = NULL;
	List *i; 

	if(!(*list_ptr)){
		*list_ptr = new_list;
		/*We want to update the tail_ptr*/
		prev_list = NULL;
		return;
	}
	if(list_ptr != prev_list){
		i = *list_ptr;
		prev_list = list_ptr;
		while(i && i->next){
			i = i->next;
		}
		tail_ptr = i;
	}
	tail_ptr->next = new_list;
	i = tail_ptr;
	while(i->next){
		i = i->next;
	}
	tail_ptr = i;
}

static List * GetElementList(Element *instance){
	List *new_list = NULL;
	List *ptr1 = instance->fanout_nets;
	NetList *net_list=NULL;
	List *i;
	char buf[512];
	while(ptr1){
		assert((strlen(instance->instance_name) + strlen((char *)ptr1->_data)) < 512);
		sprintf(buf,"%s/%s",instance->instance_name,(char *)ptr1->_data);
		//if(hfetch(TFaultHash,buf)){
			/*We have to insert netlist corresponding
			 *to the net */
			assert(instance->parent_cell != NULL);
			net_list = (NetList *)hfetch((instance->parent_cell)->net_list,(char *)ptr1->_data);
			if(!net_list){
				fprintf(stderr,"NetList Error for pin %s\n",(char *)ptr1->_data);
			}
			i = net_list->net_start;
			while(i){
				if(!(((Element *)i->_data)->Marked)){
					InsertList(FIRST,&new_list,(void *)i->_data);
					((Element *)i->_data)->Marked = 1;
				}
			/*	if((Element *)i->_data != instance){
					AddPath2Element(instance,(Element *)i->_data,
					(char *)ptr1->_data);
				} */
				i = i->next;
			}
		//}
		ptr1 = ptr1->next;
	}
	return new_list;
}
void PrintPathStrings(List *ptr){

	printf("<START>\n");
	while(ptr){
		printf("%s ",(char *)ptr->_data);
		ptr = ptr->next;
	}
	printf("\n<END>\n");
}
void PrintPaths(Element *e){
	List *ptr = e->path_list;
	while(ptr){
		printf("%s\n",(char *)ptr->_data);
		ptr = ptr->next;
	}
}
char *GetPathPin(Element *e){
	int length;
	char *new_string;
	char *instance_name = e->instance_name;
	char *pin_name = hfetch(e->net_list,
		"__pathfinder__connecting_net");
	char *pin_name1 = NULL;

	pin_name1 = hfetch(e->net_list,pin_name);
	assert(instance_name != NULL);
	assert(pin_name != NULL);
	assert(pin_name1 != NULL);

	length = strlen(instance_name) + strlen(pin_name1) + strlen(pin_name);
	assert(length > 0);
	new_string = (char *)malloc(sizeof(char)*(length+9));
	assert(new_string != NULL);
	sprintf(new_string,"%s/%s [%s]",instance_name,pin_name1,pin_name);
	return new_string;
}
/*We want the path to have only combinatorial elements*/
void RecursiveFindAllPaths(Element *start,Element *end){
	static List *path_ptr = NULL;
	static List **tail_ptr = NULL;
	List **local_ptr = NULL;
	Element *e;
	List *ptr;
	/*We have reached end*/
	if(start == end){
		if(path_ptr){
			if(path_size > 1){
				printf("PATH Between [%s] -> [%s] (SIZE=%ld):\n",
					start_mem_cell_name,
					end->instance_name,path_size);
				printf("-------------\n");
				PrintPathStrings(path_ptr);
				printf("------------------\n");
			}
		}else{
			fprintf(stderr,"ERROR: RecursiveFindAllPaths: path between %sand %s is incorrect\n",start->instance_name,end->instance_name);
		}
		return;
	}

	start->Marked = 1;
	ptr = GetUnMarkedElements(start);
	while(ptr){
		e = (Element *)ptr->_data; 
		if(!path_ptr){
			tail_ptr = &path_ptr;
		}

		(*tail_ptr) = (List *)malloc(sizeof(List)*1);
		local_ptr = tail_ptr;
		(*tail_ptr)->_data = GetPathPin(e); 
		(*tail_ptr)->next = NULL;
		tail_ptr = &((*tail_ptr)->next);

		if(e == end || 
			strcmp(memory_module_name,e->mod_name)){
			path_size++;
			RecursiveFindAllPaths(e,end);
			path_size--;
		}
		
		free((*local_ptr)->_data);
		free(*local_ptr);
		tail_ptr = local_ptr;
		ptr = ptr->next;
	}
	start->Marked = 0;
}

/*
 *Takes the start_node, end_node and Transition Fault list tf_list
 *and finds all the paths between them.
 **/
void FindAllPaths(Element *start,Element *end, List *tf_list){
	List *bfs_list = NULL;
	Element *node;
	List *prev_list_node;

	if(!TFaultHash){
		CreateTFaultHash(tf_list);
	}

	start->Marked = 1;
	bfs_list = GetElementList(start); 
	while(bfs_list){
		/*Pop the first node*/
		prev_list_node = bfs_list;
		node = (Element *)bfs_list->_data;
		bfs_list = bfs_list->next;
		if(node != end){
			AppendToList(&bfs_list,GetElementList(node));
		}
		free(prev_list_node);
	}
}
