/*
 * PathFinder: Find the path's covered by the pattern in the circuit.
 * vamsi.krishnak@gmail.com	April, 2007
 */

#include <assert.h>
#include <stdio.h>
#include "hash.h"
#include "verilog.h"
#include "util.h"
char *start_mem_cell_name;
long path_size;
#ifndef NO_UNIT_TESTS
void UnitTestTopCell(char *cell_name);
void PrintAllModuleDefs();
void TestNets();
void TestPairWisePaths(char *cell_name);
#endif

extern FILE *yyin;
HASH *mod_def = NULL;
char library_mode = 0;
/*List of the memory elements*/
List *memory_element_list = NULL;
/*Name of the memory module*/
char *memory_module_name = NULL;
long memory_element_count = 0;
extern unsigned int current_line_no;
int my_strcmp(char *a,char *b){
	register int val=0;
	while(*a && *b){
		val += (*a++) - (*b++);
	}
	return val;
}


void PairWisePaths(char *cell_name){
	Module *cell_def = hfetch(mod_def,cell_name);
	assert(cell_def != NULL);
	Element *e; List *e_list = memory_element_list,*e_list1;
	assert(e_list != NULL);
	while(e_list){
		e_list1 = memory_element_list;
		while(e_list1){
			if((Element *)e_list->_data == 
				(Element *)e_list1->_data){
				e_list1 = e_list1->next;
				continue;
			}
/*			printf("<===========================>\n");
			printf("All Paths between [%s] and [%s]:\n",
				((Element *)e_list->_data)->instance_name,
				((Element *)e_list1->_data)->instance_name);
			printf("---------\n");*/
			start_mem_cell_name = ((Element *)e_list->_data)->instance_name;
			path_size = 0;
			RecursiveFindAllPaths((Element *)e_list->_data,
				(Element *)e_list1->_data);
		/*	printf("\n<===========================>\n\n");*/
			e_list1 = e_list1->next;
		}
		e_list = e_list->next;
	}
}




Module* CreateNewModule(char *mod_name,List *pin_list, Module *parent_cell){
	Module *ptr = NULL;
	List *ptr1 = pin_list;
	List *ptr1_prev = NULL;
	char msg_buf[128];
	HENT *entry = NULL;

	if(!mod_def){
		mod_def = hnew();
	}
	/*Check for re-definition*/
	if(hfetch(mod_def,mod_name)){
		fprintf(stderr,"CreateNewModule: %s Module Redefined",mod_name);
		exit(1);
	}
	ptr = (Module *) malloc(sizeof(Module)*1);
	assert(ptr!=NULL);
	current_module = ptr;
	if(!pin_list){
		sprintf(msg_buf,"CreateNewModule: module %s with no pins",mod_name);
		HandleParseError(msg_buf);
	}
#if PF_VERBOSE
	printf("Stored %s\n",mod_name);	
#endif

	hstore(mod_def,mod_name,(void *)ptr);
	ptr->nets = hnew();
	ptr->net_list = hnew();
	assert(ptr->nets != NULL && ptr->net_list != NULL);
	while(ptr1){
		if(hfetch(ptr->nets,(char *)ptr1->_data)){
			sprintf(msg_buf,"CreateNewModule: Port %s Redefined in module %s",ptr1->_data,mod_name); 
			HandleParseError(msg_buf);
		}
		hstore(ptr->nets,(char *)ptr1->_data,"ARG");
		if(!hfetch(ptr->nets,(char *)ptr1->_data)){
			printf("Storing Failed\n");
		}
		ptr1_prev = ptr1;
		ptr1 = ptr1->next;
		if(!ptr1_prev){
			free(ptr1_prev);
		}
	}
	ptr->fanin_nets = NULL;
	ptr->fanout_nets = NULL;
	ptr->elements = NULL;
	ptr->path_list = NULL;
	ptr->Marked = 0;
	ptr->parent_cell = parent_cell;
	return ptr;
}

char *strdup_verilog(char *str,size_t len){
	char *ptr = NULL;
	ptr = (char *) malloc(sizeof(char)*(len+1));
	assert(ptr!=NULL);
	ptr[len] = '\0';
	if(memcpy(ptr,str,len)!=ptr){
		fprintf(stderr,"strdup_verilog: memcpy failed");
		return NULL;
	}
	return ptr;
}
/*Set the Type to type for the given nets in the list*/
char SetPinType(Module *mod, List *nets,char type){
	List *ptr = nets;
	char *val = NULL;
	unsigned int count=0;
	while(ptr){
		val = hfetch(mod->nets,(char *)ptr->_data);
		if(!val){
			fprintf(stderr,"SetPinType: Unable to find pin %s in the argument list of the module %s\n",
			(char *)ptr->_data,mod->mod_name);
			return FAILURE;
		}
		count++;
		if(type == OUT_PIN){
			hstore(mod->nets,(char *)ptr->_data,"OUTPUT");
		}else{
			hstore(mod->nets,(char *)ptr->_data,"INPUT");
		}
		ptr = ptr->next;
	}
	if(type == OUT_PIN){
		mod->fanout += count;
		mod->fanout_nets = nets;
	}else{
		mod->fanin += count;
		mod->fanin_nets = nets;
	}
	return SUCCESS;
}

char AddElementToModule(Element *e,Module *ptr){
	assert(e!=NULL);
	InsertList(FIRST,&(ptr->elements),e);
}

/*Free up the structures if we are going to quit*/
void HandleParseError(char *err_str){
	fprintf(stderr,"PathFinder:");
	fprintf(stderr,err_str);
}
status_t OpenVerilogFile(char *file_name){
	FILE *ptr = fopen(file_name,"r");
	if(!ptr){
		verilog_error("OpenLibFile: Error Opening File\n")
	}
	yyin = ptr;
}
/*Check Def: Checks if the definition of the module exists*/
char CheckDef(char *mod_name){
	return (hfetch(mod_def,mod_name))?1:0;
}

char *expand_this_cell = NULL;
int main(int argc,char *argv[]){
	if(argc < 6){
		verilog_error("USAGE: pathfinder {cell_definitions} {synthesized_cell_file} {top_cell_which_needs_expansion} {TFault_File} {MemoryCellModuleName}\n");
	}



	if(!OpenVerilogFile(argv[1])){
		exit(1);
	}
	memory_module_name = argv[5];
	/*Parse the LibFile*/
	libmode = 1;
	yyparse();
	fclose(yyin);
	/*Check if the module definition of the 
	 *memory element exists
	 */
	if(!CheckDef(argv[5])){
		fprintf(stderr,"Unable to find the definition of %s\n",
			argv[5]);
		exit(1);
	}



	if(!OpenVerilogFile(argv[2])){
		exit(1);
	}
	expand_this_cell = argv[3];
	printf("Parsing the cell file\n");
	libmode = 0;
	current_line_no = 0;
	yyparse();
	fclose(yyin);
	printf("PATHFINDER: A total of %ld memory elements found\n",
		memory_element_count);

	if(!OpenTFaultFile(argv[4])){
		exit(1);
	}
	CreateTFaultHash_File(TFaultFile);
	PairWisePaths(argv[3]);

#if 0
	PrintAllModuleDefs();
	UnitTestTopCell(argv[3]);
  	TestNets(argv[3]);
  	TestPairWisePaths(argv[3]);
#endif

}

Element * CreateNewElement(char *inst_name,Module *def,Module *parent_cell){
	Element *ptr = NULL;
	ptr = (Element *) malloc(sizeof(Element)*1);
	ptr->instance_name = inst_name;
	ptr->mod_name = def->mod_name;
	/*Check if this is the memory element*/
	if(!strcmp(memory_module_name,ptr->mod_name)){
		memory_element_count++;
		InsertList(FIRST,&(memory_element_list),(void *)ptr);
	}
	ptr->parent_cell = parent_cell;
	ptr->path_list = NULL;
	ptr->fanin_nets = NULL;
	ptr->fanout_nets = NULL;
    ptr->path_list = NULL;	
	ptr->Marked = 0;
	ptr->net_list = hnew();
	assert(ptr->net_list != NULL);
	ptr->fanin = 0;
	ptr->fanout = 0;
	ptr->nets = NULL;
}
/*Unit Test Case*/

#ifndef NO_UNIT_TESTS
void UnitTestTopCell(char *cell_name){
	Module *top_cell = (Module *)hfetch(mod_def,cell_name);
	Element *e;
	List *e_list,*e_fanin,*e_fanout;
	printf("TopCell=%s\n",cell_name);
	assert(top_cell != NULL);
	e_list = top_cell->elements;
	while(e_list){
		e = (Element *) e_list->_data;
		e_fanin = e->fanin_nets;
		
		printf("Fanin Nets for Instance ::%s of Type::%s\n",e->instance_name,e->mod_name);
		while(e_fanin){
			printf("%s ",(char *)e_fanin->_data);
			e_fanin = e_fanin->next;
		}
		printf("\n");


		printf("Fanout Nets for Instance ::%s of Type::%s\n",e->instance_name,e->mod_name);
		e_fanout = e->fanout_nets;
		while(e_fanout){
			printf("%s ",(char *)e_fanout->_data);
			e_fanout = e_fanout->next;
		}
		printf("\n");

		e_list = e_list->next;
	}
}

void PrintAllModuleDefs(){
	int i;
	HENT *entry;
	hiterinit(mod_def);
	for(i=0;entry=(HENT *)hiternext(mod_def);i++){
		printf("%s --> %lx\n",hiterkey(entry),(void *)hiterval(entry));
	}
}

void TestNets(char *cell_name){
	char buffer[256];
	Module *def = hfetch(mod_def,cell_name);
	NetList *net_list = NULL;
	List *l;

	assert(def!=NULL);
	while(1){
		if(scanf("%s",buffer) < 1){ 
			break;
		}
		/*Query the net, The netlist has
		 *the Element
		 */
		net_list = hfetch(def->net_list,buffer);
		assert(net_list != NULL);
		l = net_list->net_start;
		printf("The following are the nets connected to %s\n",
			buffer);
		while(l){
			printf("%s->",((Element *)l->_data)->instance_name);
			l = l->next;
		}
		printf("\n");
	}
	printf("Thank you.....\n");
}


void TestPairWisePaths(char *cell_name){
	Module *cell_def = hfetch(mod_def,cell_name);
	assert(cell_def != NULL);
	Element *e; List *e_list = cell_def->elements,*e_list1;
	assert(e_list != NULL);
	while(e_list){
		e_list1 = cell_def->elements;
		while(e_list1){
			if((Element *)e_list->_data == 
				(Element *)e_list1->_data){
				e_list1 = e_list1->next;
				continue;
			}
			start_mem_cell_name = ((Element *)e_list->_data)->instance_name;
/*			printf("All Paths between %s and %s\n",
				((Element *)e_list->_data)->instance_name,
				((Element *)e_list1->_data)->instance_name);*/
			RecursiveFindAllPaths((Element *)e_list->_data,
				(Element *)e_list1->_data);
			e_list1 = e_list1->next;
		}
		e_list = e_list->next;
	}
}
#endif
