/**
 * Yacc file for parsing the technology mapped netlist 
 * from the design compiler.
 * vamsi.krishnak@gmail.com Mar 29 2007
 */
%{
#include "List.h"
#include "hash.h"
#include<assert.h>
#include "verilog.h"
static List *arglist = NULL;
static List *argptr = NULL;
%}
%union {
	char *str;
	int num;
	List *list;
	PinPair *ppair;
}
%token <str> WORD
%token <str> IDENTIFIER
%token <str> MODULE_KEY_WORD
%token <str> ENDMODULE_KEY_WORD
%token <str> OUTPUT_KEY_WORD
%token <str> INPUT_KEY_WORD
%token <str> STRING /*any thing except a ';'*/
%token <str> SEMI_COLON
%token <str> DOT
%token <str> LEFT_BRACE
%token <str> RIGHT_BRACE
%token <str> COMMA
/*Type definitions for the values of the rules*/
%type <list> termlist
%type <list> arguments
%type <list> pinpairarguments
%type <ppair> pinpair
%type <list> arglist
%%
verilog: verilog module | module  
module: modsignature modbody modend
modsignature: MODULE_KEY_WORD IDENTIFIER arglist SEMI_COLON 
			{
				Module *m = CreateNewModule($2,$3);
				m->mod_name = $2;
			}
arglist: | LEFT_BRACE termlist RIGHT_BRACE 
		{
			$$ = $2;
		}

termlist: pinpairarguments 
		  {
			  List *ptr = $1;
			  PinPair *ptr1;
			  while(ptr){
				  ptr1 = (PinPair *) ptr->_data;
				  printf("PinPair .%s(%s)\n",
				  	ptr1->block_pin,ptr1->net_pin);
				  ptr = ptr->next;
			  }
			  $$ = $1;
		  }
		  | arguments 
		  { 
			  printf("Printing the ARGLIST.......\n");
			  PrintList_String($1);
			  $$ = $1;
		  }

pinpair: DOT IDENTIFIER LEFT_BRACE IDENTIFIER RIGHT_BRACE 
		{
			PinPair *ptr = (PinPair *) malloc(sizeof(PinPair)*1);
			ptr->block_pin = $2;
			ptr->net_pin = $4; 
			$$ = ptr;
		}
pinpairarguments: pinpair 
				  {
			  		argptr = NewListNode();
			  		argptr->_data = $1; 
			  		argptr->next = NULL;
			  		$$ = argptr;
				  } 
				 | pinpair COMMA pinpairarguments 
				 {
			  		argptr = NewListNode();
			  		argptr->_data = $1;
			  		argptr->next = $3;
			  		$$ = argptr;
				 }
arguments: IDENTIFIER 
		   {
			  argptr = NewListNode();
			  argptr->_data = (void *)$1; 
			  argptr->next = NULL;
			  $$ = argptr;
		   } 
	       | IDENTIFIER COMMA arguments
		   {
			  argptr = NewListNode();
			  argptr->_data = $1;
			  argptr->next = $3;
			  $$ = argptr;
		   }
modend: ENDMODULE_KEY_WORD

/*A module must at atleast 1 input and 1 output statement*/
modbody: statements inputstatement statements outputstatement statements
statements: | statementline statements
inputstatement: INPUT_KEY_WORD arguments SEMI_COLON {
}
outputstatement: OUTPUT_KEY_WORD arguments SEMI_COLON { 
}
statementline: netliststatements SEMI_COLON | nonnetliststatement
nonnetliststatement: STRING {
	printf("A non-netlist statement found\n");
}
netliststatements: gatestatement 
				  | nongatestatement
nongatestatement:IDENTIFIER arguments{
	
}

gatestatement: IDENTIFIER IDENTIFIER '(' arguments ')' {
	
}
%%
#include<stdio.h>
int main(){
	yyparse();	
}

int yywrap(){
	return 1;
}
extern unsigned int current_col_no;
extern unsigned int current_line_no;

int yyerror(char *yymsg){
	fprintf(stderr,"Parse error....%s",yymsg);
	fprintf(stderr,"At line #%ld-%ld\n",current_line_no,current_col_no);
}

List *NewListNode(){
	List *ptr = NULL;
	ptr = (List *) malloc(sizeof(List)*1);
	assert(ptr != NULL);
	return ptr;
}
